// $Id: popups.js,v 1.9.8.4 2009/03/05 21:12:26 starbow Exp $

/**
 * Popup Modal Dialog API
 *
 * Provide an API for building and displaying JavaScript, in-page, popups modal dialogs.
 * Modality is provided by a fixed, semi-opaque div, positioned in front of the page contents.
 *
 */

/**
 * Create the popups object/namespace.
 */
Drupal.popups = function() {};

Drupal.popups.isset = function(v) {
  return (typeof(v) !== 'undefined');
}

/**
 * Attach the popups bevior to the all the requested links on the page.
 *
 * @param context
 *   The jQuery object to apply the behaviors to.
 */
Drupal.behaviors.popups = function(context) {
  var $body = $('body');
  if(!$body.hasClass('popups-processed')) {
    $body.addClass('popups-processed');
    $(document).bind('keydown', Drupal.popups.keyHandle);
    var $popit = $('#popit');
    if ($popit.length) {
      $popit.remove();
      Drupal.popups.message($popit.html());
    }
  }
  
  // Add the popups-link-in-dialog behavior to links defined in Drupal.settings.popups.links array.
  if (Drupal.settings.popups.links) {
    jQuery.each(Drupal.settings.popups.links, function (link, options) { 
      if (Drupal.popups.isset(options.noReload)) { // Using obsolete name.
        options.noUpdate = options.noReload; // Don't break existing sites with name change.
      }
      Drupal.popups.attach(context, link, options);
    });
  }
  
  Drupal.popups.attach(context, '.popups', {noUpdate: true});  
  Drupal.popups.attach(context, '.popups-form', {}); // ajax reload.
  Drupal.popups.attach(context, '.popups-form-reload', {reloadWhenDone: true}); // whole page reload. 
  Drupal.popups.attach(context, '.popups-form-noupdate', {noUpdate: true});  // no reload at all.
  Drupal.popups.attach(context, '.popups-form-noreload', {noUpdate: true});  // Obsolete.
  
};

/**
 * Attach the popups behavior to a particular link.
 *
 * @param selector
 *   jQuery selector for links to attach popups behavior to.
 * @param options
 *   Hash of options associated with these links.
 */
Drupal.popups.attach = function(context, selector, options) {
  $(selector, context).not('.popups-processed').each(function() {
    var $element = $(this);
    // Mark the element as attached.    
    var title = $element.attr('title') || '';
    $element.attr('title', title + Drupal.t('[Popup]')); // Append note to link title.
    $element.addClass('popups-processed');
    
    // Attach the on-click popup behavior to the element.
    $element.click(function(e){ 
      var element = this;

      // If element is inside of a #popup div, show alert and bail out. 
      if ($(element).parents('#popups').length) { 
        alert("Sorry, popup chaining is not supported (yet).");
        return false;
      }

      // If the element contains a on-popups-options attribute, use it instead of options param.
      if ($(element).attr('on-popups-options')) {
        options = eval('(' + $(element).attr('on-popups-options') + ')'); 
      }

      // If the option is distructive, check if the page is already modified, and offer to save.
      var pageIsDirty = $('span.tabledrag-changed').size() > 0;
      var willModifyOriginal = !options.noUpdate;
      if (pageIsDirty && willModifyOriginal) {
        // The user will lose modifications, so popups dialog offering to save current state.
        var body = Drupal.t("There are unsaved changes on this page, which you will lose if you continue.");
        var buttons = {
         'popup_save': {title: Drupal.t('Save Changes'), func: function(){Drupal.popups.savePage(element, options);}},
         'popup_submit': {title: Drupal.t('Continue'), func: function(){Drupal.popups.removePopup(); Drupal.popups.openPath(element, options);}},
         'popup_cancel': {title: Drupal.t('Cancel'), func: Drupal.popups.close}
        };
        return Drupal.popups.open( Drupal.t('Warning: Please Confirm'), body, buttons );
      }
      else {
        return Drupal.popups.openPath(element, options);
      } 
    });    
  });
};


/**
 * Generic dialog builder.
 * TODO: capture the focus when it leaves the dialog.
 */
Drupal.popups.open = function(title, body, buttons, width) {
  Drupal.popups.addOverlay(); // TODO - nonModal option.
  var $popups = $(Drupal.theme('popupDialog', title, body, buttons));
  // Start with dialog off the side. Making it invisible causes flash in FF2.
  $popups.css('left', '-9999px');
  if (width) {
    $popups.css('width', width);
  }
  $('body').append( $popups ); // Add the popups to the DOM.

  // Adding button functions
  if (buttons) {
    jQuery.each(buttons, function (id, button) { 
      $('#'+id).click(button.func);
    });  
  }
  $('#popups-close').click(Drupal.popups.close);
  $('a.popups-close').click(Drupal.popups.close);
    
  // center on the screen, adding in offsets if the window has been scrolled
  var popupWidth = $popups.width();  
  var windowWidth = Drupal.popups.windowWidth();
  var left = (windowWidth / 2) - (popupWidth / 2) + Drupal.popups.scrollLeft();
  
  // Get popups's height on the page.
  // Need to do it this way, since we get visible flash in FF2 if popups is not visible!
  var popupHeight = $popups.height(); 
  var windowHeight = Drupal.popups.windowHeight();
  if ($.browser.opera && $.browser.version > "9.5" && $.fn.jquery <= "1.2.6") { // Fix for Opera bug - http://drupal.org/node/366093
    windowHeight = document.documentElement["clientHeight"];
  }
   
  if (popupHeight > (0.9 * windowHeight) ) { // Must fit in 90% of window.
    popupHeight = 0.9 * windowHeight;
    $popups.height(popupHeight);
  }  
  var top = (windowHeight / 2) - (popupHeight / 2) + Drupal.popups.scrollTop();

  $popups.css('top', top).css('left', left); // Position the popups to be visible.

// Dynamic iframe  
//  setTimeout( function() {
//    var doc = $('#popups-body')[0].contentWindow.document;
//    var $body = $('body',doc);
//    $body.html(body);
//  }, 1 ); 
  
  // Focus on the first input element in the popup window.
  this.refocus(); 
  
  // Remove the loading image.
  Drupal.popups.removeLoading();
   
  return false;
};

/**
 *  Simple popups that functions like the browser's alert box.
 */
Drupal.popups.message = function(title, message) {
  message = message || '';
  var buttons = {
    'popup_ok': {title: Drupal.t('OK'), func: Drupal.popups.close}
  };
  Drupal.popups.open(title, message, buttons);
};

/**
 * Handle any special keys when popups is active.
 */
Drupal.popups.keyHandle = function(e) {
  if (!e) {
    e = window.event;
  }
  switch (e.keyCode) {
    case 27: // esc
      Drupal.popups.close();
      break;
    case 191: // '?' key, show help.
      if (e.shiftKey && e.ctrlKey) {
        var $help = $('a.popups.more-help');
        if ($help.size()) {
          $help.click();
        }
        else {
          Drupal.popups.message(Drupal.t("Sorry, there is no additional help for this page"));
        }
      }
      break;
  }
};

/*****************************************************************************
 * Appearence Functions (overlay, loading graphic, remove popups)     *********
 *****************************************************************************/

Drupal.popups.removePopup = function() {
  $('#popups').remove();  
}; 
 
/**
 * Add full page div between the page and the dialog, to make the popup modal.
 */
Drupal.popups.addOverlay = function() {
  var $overlay = $('#popups-overlay');
  if (!$overlay.length) { // Overlay does not already exist, so create it.
    $overlay = $(Drupal.theme('popupOverlay'));
    $overlay.css('opacity', '0.4'); // for ie6(?)
    // Doing absolute positioning, so make overlay's size equal the entire body.
    var $doc = $(document);
    $overlay.width($doc.width()).height($doc.height()); 
    $overlay.click(Drupal.popups.close);
    $('body').prepend($overlay);
  }
};

Drupal.popups.removeOverlay = function() {
  $('#popups-overlay').remove();
};

/**
 * Add a "Loading" message while we are waiting for the ajax response.
 */
Drupal.popups.addLoading = function() {
  var $loading = $('#popups-loading');
  if (!$loading.length) { // Loading image does not already exist, so create it.
    $loading = $(Drupal.theme('popupLoading'));
    $('body').prepend($loading); // Loading div is initially display:none.
    var width = $loading.width();
    var height = $loading.height();
    var left = (Drupal.popups.windowWidth() / 2) - (width / 2) + Drupal.popups.scrollLeft();
    var top = (Drupal.popups.windowHeight() / 2) - (height / 2) + Drupal.popups.scrollTop();
    $loading.css({'top': top, 'left': left, 'display': 'block'}); // Center it and make it visible.
  }
};

Drupal.popups.removeLoading = function() {
  $('#popups-loading').remove();
};

/**
 * Remove everything.
 */
Drupal.popups.close = function() {
  Drupal.popups.removePopup();
  Drupal.popups.removeLoading();
  Drupal.popups.removeOverlay();  
  Drupal.popups.restorePage();
  return false;
};

/**
 * Remove as much of the effects of jit loading as possible.
 */
Drupal.popups.restorePage = function() {
//  console.log("Restoring Drupal.settings");
  // Restore the page's original Drupal.settings.
  Drupal.settings = Drupal.popups.originalSettings; 
  // Remove the CSS files that were jit loaded for popup.
  for (var i in Drupal.popups.newCss) {
    var link = Drupal.popups.newCss[i];
//    console.log("Removing " + link);
    $('link[href='+ $(link).attr('href') + ']').remove();
  }
};

/**
 * Set the focus on the popups to the first visible, enabled form element, or the close link.
 */
Drupal.popups.refocus = function() {
  // Select the first visible enabled input element.
  var $focus = $('#popups :input:visible:enabled:first');
  if (!$focus.length) {
    // There is no visible enabled input element, so select the close link.
    $focus = $('#popups-close a');
  }
  $focus.focus();
};

/****************************************************************************
 * Theme Functions   ********************************************************
 ****************************************************************************/

Drupal.theme.prototype.popupLoading = function() {
  var loading = '<div id="popups-loading">';
  loading += '<img src="'+ Drupal.settings.basePath + Drupal.settings.popups.modulePath + '/ajax-loader.gif" />';
  loading += '</div>';
  return loading;
};

Drupal.theme.prototype.popupOverlay = function() {
  return '<div id="popups-overlay"></div>';
};

Drupal.theme.prototype.popupButton = function(title, id) {
  return '<input type="button" value="'+ title +'" id="'+ id +'" />';
};

Drupal.theme.prototype.popupDialog = function(title, body, buttons) {
  var template = Drupal.theme('popupTemplate');
  var popups = template.replace('%title', title).replace('%body', body);
  
  var themedButtons = '';
  if (buttons) {
    jQuery.each(buttons, function (id, button) { 
      themedButtons += Drupal.theme('popupButton', button.title, id);
    });  
  }  
  popups = popups.replace('%buttons', themedButtons);  
  return popups;
};

Drupal.theme.prototype.popupTemplate = function() {
  var template;
  template += '<div id="popups">';
  template += '  <div id="popups-title">';
  template += '    <div id="popups-close"><a href="#">' + Drupal.t('close') + '</a></div>';
  template += '    <div class="title">%title</div>';
  template += '    <div class="clear-block"></div>';
  template += '  </div>';
  template += '  <div id="popups-body">%body</div>';
//  template += '  <iframe id="popups-body" name="popups-body">Here is some content</iframe>';
  template += '  <div id="popups-buttons">%buttons</div>';
  template += '  <div id="popups-footer"></div>';
  template += '</div>';
  return template;
}

/**
 * Get the position of the left side of the browser window.
 */
Drupal.popups.scrollLeft = function() {
  return Math.max(document.documentElement.scrollLeft, document.body.scrollLeft);
};

/**
 * Get the position of the top of the browser window.
 */
Drupal.popups.scrollTop = function() {
  return Math.max(document.documentElement.scrollTop, document.body.scrollTop);
};

/**
 * Get the height of the browser window.
 * Fixes jQuery & Opera bug - http://drupal.org/node/366093
 */
Drupal.popups.windowHeight = function() {
  if ($.browser.opera && $.browser.version > "9.5" && $.fn.jquery <= "1.2.6") { 
    return document.documentElement["clientHeight"];
  }
  return $(window).height();
};

/**
 * Get the height of the browser window.
 * Fixes jQuery & Opera bug - http://drupal.org/node/366093
 */
Drupal.popups.windowWidth = function() {
  if ($.browser.opera && $.browser.version > "9.5" && $.fn.jquery <= "1.2.6") { 
    return document.documentElement["clientWidth"];
  }
  return $(window).width();
};

/****************************************************************************
 * Page & Form in popups functions                                         ***
 ****************************************************************************/

/**
 * Use Ajax to open the link in a popups window.
 *
 * @param element
 *   Element that was clicked to open the popups.
 * @param options
 *   Hash of options controlling how the popups interacts with the underlying page.
 */
Drupal.popups.openPath = function(element, options) {
  // let the user know something is happening
  $('body').css("cursor", "wait");
  
  // TODO - get nonmodal working.
  if (!options.nonModal) {
    Drupal.popups.addOverlay(); 
  }
  Drupal.popups.addLoading();
  
  var href = options.href ? options.href : element.href;
  $(document).trigger('popups_open_path', [element, href]); // Broadcast Popup Open Path event.
  
  var params = {}; 
  // Force the popups to return back to the orignal page when forms are done, unless forceReturn option is set.
  if (!options.forceReturn) { // If forceReturn, requestor wants data from different page.
    href = href.replace(/destination=[^;&]*[;&]?/, ''); // Strip out any existing destination param.
    params.destination = Drupal.settings.popups.originalPath; // Set the destination to the original page.    
  }

  var ajaxOptions = {
    url: href,
    dataType: 'json',
    data: params,
    beforeSend: Drupal.popups.beforeSend,
    success: function(json) { 
      // Add additional CSS to the page.
      Drupal.popups.addCSS(json.css);
      var inlines = Drupal.popups.addJS(json.js);    
      Drupal.popups.openContent(json.title, json.messages + json.content, options, element);
      Drupal.popups.addInlineJS(inlines);      
    },
    complete: function() {
      $('body').css("cursor", "auto"); // Return the cursor to normal state.      
    }
  };

  if (options.reloadOnError) {
    ajaxOptions.error = function() {
      location.reload(); // Reload on error. Is this working?
    };    
  }
  else {
   ajaxOptions.error = function() {
      Drupal.popups.message("Unable to open: " + href);
    };
  }
  $.ajax(ajaxOptions);
        
  return false;         
};

/**
 * Add additional CSS to the page.
 */
Drupal.popups.addCSS = function(css) {
  Drupal.popups.newCss = [];
  for (var type in css) {
    for (var file in css[type]) {
      var link = css[type][file];
      //console.log("new page includes " + $(link).attr('href'));
      // Does the page already contain this stylesheet?
      if (!$('link[href='+ $(link).attr('href') + ']').length) {
        console.log( "ADDING " + link);
        $('head').append(link);
        Drupal.popups.newCss.push(link);
      }
    }
  }
}

/**
 * Add additional Javascript to the page.
 */
Drupal.popups.addJS = function(js) {
//  console.log("Loading js files");
//  console.log(js);

  var scripts = [];
  var inlines = [];
  for (var type in js) {
    if (type != 'setting') {
      for (var file in js[type]) {
        if (type == 'inline') {
          inlines.push($(js[type][file]).text());
        }
        else {
          scripts.push($(js[type][file]).attr('src'));
        }
      }
    }
  }

  // Add new JS settings to the page, needed for #ahah properties to work.
  Drupal.popups.originalSettings = Drupal.settings; // Cache orginial page settings.
  Drupal.settings = js.setting;
//  console.log ("Temp setting settings to: ");
//  console.log(Drupal.settings);
  // As an exception, keep the popups settings in place until the popup is closed.
  Drupal.settings.popups = Drupal.popups.originalSettings.popups;

  for (var i in scripts) {
    var src = scripts[i];
    if (!$('script[src='+ src + ']').length) {
//      console.log("Load js " + file);
      // Get the script from the server and execute it.
      $.ajax({ 
        type: 'GET',
        url: src,
        dataType: 'script',
        async : false,
        success: function(script) {
//          console.log("async evaling script");
          eval(script);
        }
      });
    }
  }

  return inlines;
};

Drupal.popups.addInlineJS = function(inlines) {
  // Load the inlines into the page.
  for (var n in inlines) {
    if (!$('script:not([src]):contains(' + inlines[n] + ')').length) {
//      console.log( "Eval " + inlines[n] );
      eval(inlines[n]);
    }
  }
};

/**
 * Open content in an ajax popups.
 *
 * @param title
 *   String title of the popups.
 * @param content
 *   HTML to show in the popups.
 * @param options
 *   Hash of options controlling how the popups interacts with the underlying page.
 * @param element
 *   A DOM object containing the element that was clicked to initiate the popup. 
 */
Drupal.popups.openContent = function(title, content, options, element) {
  Drupal.popups.open(title, content, null, options.width); 
  // Add behaviors to content in popups. 
  delete Drupal.behaviors.tableHeader; // Work-around for bug in tableheader.js (http://drupal.org/node/234377)
  delete Drupal.behaviors.teaser; // Work-around for bug in teaser.js (sigh).
  Drupal.attachBehaviors($('#popups-body'));
  // Adding collapse moves focus.
  Drupal.popups.refocus();

  // If the popups contains a form, capture submits.
  var $form = $('form', '#popups-body');
  $form.ajaxForm({   
    dataType: 'json',   
    beforeSubmit: Drupal.popups.beforeSubmit,
    beforeSend: Drupal.popups.beforeSend,
    success: function(response, status) {
      Drupal.popups.formSuccess(response, options, element);
    },
    error: function() {
      Drupal.popups.message("Bad Response form submission");
    }
  });
};

Drupal.popups.beforeSend = function(xhr) {
  xhr.setRequestHeader("X-Drupal-Render-Mode", 'json/popups');
};

/**
 * Do before the form in the popups is submitted.
 */
Drupal.popups.beforeSubmit = function(formData, $form, options) {
  Drupal.popups.removePopup(); // Remove just the dialog, but not the overlay.
  Drupal.popups.addLoading();
};

/**
 * The form in the popups was successfully submitted
 * Update the originating page.
 * Show any messages in a popups (TODO - make this a configurable option).
 * 
 * @param response
 *   JSON object from server with status of form submission.
 * @param options
 *   Hash of options controlling how the popups interacts with the underlying page.
 *     noUpdate: bool, does the popups effect the underlying page.
 *     nonModal: bool, does the popups block access to the underlying page.
 *     targetSelectors: hash of jQuery selectors, overrides defaultTargetSelector.
 *     titleSelectors: array of jQuery selectors, where to put the the new title of the page.
 * @param element
 *   A DOM object containing the element that was clicked to initiate the popup. 
 */
Drupal.popups.formSuccess = function(data, options, element) {  
  // Determine if we are at an end point, or just moving from one popups to another.
  var done = (data.path === Drupal.settings.popups.originalPath) || (data.path === options.forceReturn);
  if (!done) { // Not done yet, so show new page in new popups.
    Drupal.popups.removeLoading();
    Drupal.popups.openContent(data.title, data.messages + data.content, options, element);
  }
  else { // Done.
    // Execute the afterSubmit callback if available.
    if (options.afterSubmit) { 
      var afterSubmitResult = eval(options.afterSubmit +'(data, options, element)');
      if (afterSubmitResult == false) { // Give afterSubmit callback a chance to skip normal processing.
        return;
      }
    }

    if (options.reloadWhenDone) { // Force a complete, non-ajax reload of the page.
      location.reload(); 
    }
    else { // Normal, targeted ajax, reload behavior.
      // show messages in dialog and embed the results in the original page.
      var showMessage = data.messages.length && !options.noMessage;
      if (showMessage) {
        Drupal.popups.message(data.messages);
        if (Drupal.settings.popups.autoCloseFinalMessage) {
          setTimeout(Drupal.popups.close, 2500); // Autoclose the message box in 2.5 seconds.
        }
  
        // Insert the message into the page above the content.
        // Might not be the standard spot, but it is the easiest to find.
        var $next = $(Drupal.settings.popups.defaultTargetSelector);
        $next.parent().find('div.messages').remove(); // Remove the current messages.
        $next.before(data.messages);
      }
          
      // Update the content area (defined by 'targetSelectors').
      if (!options.noUpdate) { 
        Drupal.popups.testContentSelector();
        if (Drupal.popups.isset(options.targetSelectors)) { // Pick and choose what returned content goes where.
          jQuery.each(options.targetSelectors, function(t_new, t_old) {
            if(!isNaN(t_new)) {
              t_new = t_old; // handle case where targetSelectors is an array, not a hash.
            }
            var new_content = $(t_new, data.content);
            var $c = $(t_old).html(new_content); // Inject the new content into the original page.
            Drupal.attachBehaviors($c);  
          });
        }
        else { // Put the entire new content into default content area.
          var $c = $(Drupal.settings.popups.defaultTargetSelector).html(data.content);
          Drupal.attachBehaviors($c);                    
        }
      }
      
      // Update the title of the page.
      if (Drupal.popups.isset(options.titleSelectors)) {
        jQuery.each(options.titleSelectors, function() {
          $(''+this).html(data.title);
        });
      }
              
      // Done with changes to the original page, remove effects.
      Drupal.popups.removeLoading();
      if (!showMessage) { 
        // If there is not a messages popups, so remove the overlay.
        Drupal.popups.close();

        //        Drupal.popups.removeOverlay();
//        console.log ("restoring original settings");
//        Drupal.settings = Drupal.popups.originalSettings;
      }
    }
    
    // Broadcast an event that popup form was done and successful.
    $(document).trigger('popups_form_success', [element]);
    
  }  // End of updating original page.
}; 

/**
 * Submit the page and reload the results, before popping up the real dialog.
 *
 * @param element
 *   Element that was clicked to open the popups.
 * @param options
 *   Hash of options controlling how the popups interacts with the underlying page.
 */
Drupal.popups.savePage = function(element, options) {
  var target = Drupal.settings.popups.defaultTargetSelector;
  var $form = $('form', target);
  var ajaxOptions = {
    dataType: 'json',
    beforeSubmit: Drupal.popups.beforeSubmit,   
    beforeSend: Drupal.popups.beforeSend,
    success: function(response, status) { 
      // Sync up the current page contents with the submit.
      var $c = $(target).html(response.content); // Inject the new content into the page.
      Drupal.attachBehaviors($c);
      // The form has been saved, the page reloaded, now safe to show the link in a popup.
      Drupal.popups.openPath(element, options); 
    } 
  };
  $form.ajaxSubmit(ajaxOptions); // Submit the form. 
};

/**
 * Warn the user if ajax updates will not work
 *   due to mismatch between the theme and the theme's popup setting.
 */
Drupal.popups.testContentSelector = function() {
  var target = Drupal.settings.popups.defaultTargetSelector;
  var hits = $(target).length;
  if (hits !== 1) { // 1 is the corrent answer.
    var msg = Drupal.t('The popup content area for this theme is misconfigured.') + '\n';
    if (hits === 0) {
      msg += Drupal.t('There is no element that matches ') + '"' + target + '"\n';
    }
    else if (hits > 1) {
      msg += Drupal.t('There are multiple elements that match: ') + '"' + target + '"\n';
    }
    msg += Drupal.t('Go to admin/build/themes/settings, select your theme, and edit the "Content Selector" field'); 
    alert(msg);
  }
};
